from .torch import acceleration_device
from .torch import parameter_count

from .files import duplicate
from .files import funnel

from .seed import seed_python
from .seed import seed_torch
from .seed import seed_numpy
from .seed import full_seed



def seed_torch(seed: int):
    import torch

    torch.manual_seed(seed)
    torch.cuda.manual_seed_all(seed)
    torch.backends.cudnn.deterministic = True
    torch.backends.cudnn.benchmark = False


def seed_numpy(seed: int):
    import numpy

    numpy.random.seed(seed)


def seed_python(seed: int):
    import random

    random.seed(seed)


def full_seed(seed: int):
    seed_python(seed)
    seed_numpy(seed)
    seed_torch(seed)



import pickle


def _save_thing(thing, file_location: str) -> bool:
    try:
        with open(file_location, "wb") as file_handler:
            pickle.dump(thing, file_handler)
        return True
    except:
        return False


def _load_thing(file_location: str):
    try:
        with open(file_location, "rb") as file_handler:
            return pickle.load(file_handler)
    except:
        return None


def funnel(file_location: str, thing: object | None = None):
    if thing is None:
        return _load_thing(file_location)

    return _save_thing(thing=thing, file_location=file_location)


def duplicate(from_location, to_location, **replacements):
    with open(from_location, "r") as f:
        content = f.read()

    for key, value in replacements.items():
        content = content.replace(f"%{key}%", value)

    with open(to_location, "w") as f:
        f.write(content)
